
import 'dart:convert' show utf8;
import 'dart:typed_data';

class ByteWriter {
  var _buffer = Uint8List(100000);
  var _index = 0;

  int get bufferIndex => _index;
  int get bufferLength => _buffer.length;

  void copy(Uint8List other){
    for (var i = 0; i < _index; i++) {
      other[i] = _buffer[i];
    }
  }

  void setBufferSize(int size){
    _buffer = Uint8List(size);
  }

  void writeBool(bool value){
    writeByte(value ? 1 : 0);
  }

  void writeString(String value){
    writeInt(value.length);
    if (value.length == 0) return;
    final encoded = utf8.encode(value);
    for (final character in encoded){
      writeByte(character);
    }
  }

  /// A positive int can be up to 256^2
  void writePositiveInt(num value){
    writeByte(value ~/ 256);
    writeByte(value.toInt() % 256);
  }

  void writeInt(num value){
    assert(value < 10000);
    assert(value > -10000);
    final abs = value.toInt().abs();
    writeByte((value >= 0 ? 100 : 0) + abs ~/ 100);
    writeByte(abs % 100);
  }

  void writeByte(int value){
    assert(value <= 256);
    assert(value >= 0);
    _buffer[_index++] = value;
  }
}